/* Floppy Disk Support Header */

#ifndef HEADER_DISK_H
#define HEADER_DISK_H

#include <inttypes.h>

#include "ubee512.h"

// Disk image record information (adjust filler for 512 bytes)
struct diski_s
{
 char id[32];		// disk image ID
 char ver[16];		// disk image version
 char type[32];		// disk image type

 uint16_t wrprot;	// write protected
 uint16_t tracks;	// number of tracks
 uint16_t heads;	// number of heads (sides)
 uint16_t secsize;	// bytes per sector
 uint16_t sectrack;	// sectors per track
 uint16_t datatrack;	// start of data tracks (after system)
 uint16_t systsecofs;	// offset to apply to system sectors
 uint16_t datasecofs;	// offset to apply to data sectors
 uint8_t skewsa[3];	// skew algorithm info for system tracks
 uint8_t skewda[3];	// skew algorithm info for data tracks
 uint8_t skewsd[128]; 	// skew data for system tracks
 uint8_t skewdd[128]; 	// skew data for data tracks
 uint16_t secterrs;	// size of sector error table (0 if not used) 
 char fill[152];	// filler to make structure 512 bytes long
};

struct disk_s
{
 FILE *fdisk;
 int wrprot;
 int itype;		// image type 0=raw, 10=dsk, 20=dip
 struct diski_s imagerec;
};

// dsk disk structure, first 0x100 bytes is header and second 0x100 bytes is
// header of the first track.  We use that to figure out the SPT and BPS for
// all other tracks on the disk.
struct dsk_s
{
 char fill_1[0x30];
 uint8_t tracks;
 uint8_t heads;
 char fill_2[0x100-0x32];
 char fill_3[0x14];
 uint8_t bps;
 uint8_t spt; 
 char fill_4[0x100-0x16]; 
};

typedef struct dsk_s dsk_t;

typedef struct disk_s disk_t;

int disk_open(const char *name, disk_t *disk);
void disk_close(disk_t *disk);
int disk_create(const char *name, disk_t *disk);

int disk_read(disk_t *disk, char *buf, int max_size, int side, int track, int sect);
int disk_write(disk_t *disk, char *buf, int size, int side, int track, int sect);

/* int disk_getsecthdr(...) ?? */
int disk_iswrprot(disk_t *disk);

#endif  /* HEADER_DISK_H */
