//******************************************************************************
//*                                  uBee512                                   *
//*             An emulator for the Premium Microbee 512K FDD model            *
//*                                                                            *
//*                               Z80 PIO module                               *
//*                                                                            *
//*                       Copyright (C) 2007 Stewart Kay                       *
//******************************************************************************
//
// This file is intended for handling the PIO low levels and to make calls to
// other device functions contained in other dedicated source files.  The
// support files if used are:
//
// printer.c		Parallel printer port
// tape.c		Tape in/out port
// serial.c		Serial in/out port
// sound.c		Speaker sound
// rtc.c		RTC interrupt
//
// To do:
//    - Send parallel printer data to file (and or local host printer)
//    - Tape out to and in from file using z80 cycle timing.
//    - Serial out to file using z80 cycle timing.
//
//==============================================================================
// ChangeLog (most recent entries are at top)
//==============================================================================
// v1.00 - 23 June 2007, S.J.Kay
// - Created a new file to implement the Z80 PIO emulation.

//==============================================================================
/*
 *  uBee512 - An emulator for the Premium Microbee 512K FDD model
 *  Copyright (C) 2007  Stewart Kay
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/ 
//==============================================================================

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "SDL.h"
#include "pio.h"
#include "sound.h"
#include "ubee512.h"
#include "mz80.h"

static UINT8 pio_porta;
static UINT8 pio_portb;

int counter;

extern FILE *debuglog;
extern int debuglvl;

//==============================================================================
// PIO Initialise
//
//   pass: void
// return: int				0 if success, -1 if error
//==============================================================================
int pio_init (void)
{
#ifdef use_audio
 return sound_init();
#endif
 return 0;
}

//==============================================================================
// PIO de-initialise
//
//   pass: void
// return: int				0
//==============================================================================
int pio_deinit (void)
{
#ifdef use_audio
 sound_deinit();
#endif
 return 0;
}

//==============================================================================
// PIO reset
//
//   pass: void
// return: int				0
//==============================================================================
int pio_reset (void)
{
#ifdef use_audio
 sound_reset();
#endif
 return 0;
}

//==============================================================================
// PIO read - Port function
//
//   pass: UINT16 port
//         struct z80PortRead *port_s
// return: UINT16		register data
//==============================================================================
UINT16 pio_r (UINT16 port, struct z80PortRead *port_s)
{
#if 0
 printf("pio_r port 0x%02x\n", port);
#endif

 if (debuglvl > 0)
    fprintf(debuglog, "pio_r port 0x%02x\n", port);

 if ((port & 0x03) == 0x00)
    return pio_porta;

 if ((port & 0x03) == 0x02)
    return pio_portb;

 return 0xFFFF;
}

//==============================================================================
// PIO write - Port function
//
//   pass: UINT16 port
//         UINT8 data
//         struct z80PortWrite *port_s
// return: void
//==============================================================================
void pio_w (UINT16 port, UINT8 data, struct z80PortWrite *port_s)
{
#if 0
 printf("counter=%d  pio_w port 0x%02x = 0x%02x\n", counter, port, data);
 counter++;
#endif

 if (debuglvl > 0)
    fprintf(debuglog, "pio_w port 0x%02x = 0x%02x\n", port, data);

 if ((port & 0x03) == 0x00)
    {
     pio_porta = data;
     return;
    } 

 if ((port & 0x03) == 0x02)
    {
     pio_portb = data;
#ifdef use_audio
     sound_w(data);
#endif
    }
}
